require 'forwardable'

module Supai
  class OrdersResponse < Resource
    extend Forwardable
    include Enumerable

    attr_accessor :total_orders
    attr_collections orders: Order

    def_delegators :orders, :each

    def initialize(hash, chain_id, store_id, params)
      set_attributes(hash)
      @chain_id = chain_id
      @store_id = store_id
      @params = params
    end

    def params
      @params
    end

    def next(token:, api: API.new)
      skip = (@params[:skip] || 0) + count
      return nil if skip >= total_orders

      @params[:skip] = skip
      Order.all(chain_id: @chain_id, store_id: @store_id, params: @params, token: token, api: api)
    end

    def inspect
      "#<#{self.class} count: #{count}}>"
    end
  end
end
