require 'cgi'
require 'time'

module Supai
  class Resource
    def self.attributes
      @json_attrs
    end

    def self.attr_accessor(*attrs)
      @json_attrs ||= []
      @json_attrs += attrs
      super
    end

    def self.attr_objects(hash)
      @json_attrs ||= []
      hash.each do |name, klass|
        @json_attrs << name
        attr_reader(name)
        define_method("#{name}=") { |obj|
          instance_variable_set("@#{name}", parse_object(obj, klass))
        }
      end
    end

    def self.attr_collections(hash)
      @json_attrs ||= []
      hash.each do |name, klass|
        @json_attrs << name
        attr_reader(name)
        define_method("#{name}=") { |ary|
          instance_variable_set("@#{name}", parse_collection(ary, klass))
        }
      end
    end

    def self.attr_timestamp_accessor(*attrs)
      @json_attrs ||= []
      attrs.each do |name|
        @json_attrs << name
        attr_reader(name)
        define_method("#{name}=") { |value|
          instance_variable_set("@#{name}", parse_timestamp(value))
        }
      end
    end

    def as_json
      self.class.attributes.to_h { |attr|
        value = send(attr)
        json_value = case
                     when value.respond_to?(:as_json)
                       value.as_json
                     when value.is_a?(Array)
                       value.map(&:as_json)
                     else
                       value
                     end
        [attr, json_value]
      }
    end

    def underscore(value)
      return value unless /[A-Z-]/.match?(value)
      word = value.to_s.gsub(/([A-Z\d]+)([A-Z][a-z])/, '\1_\2')
      word.gsub!(/([a-z\d])([A-Z])/, '\1_\2')
      word.tr!("-", "_")
      word.downcase!
      word
    end

    def set_attribute(key, value)
      setter = "#{underscore(key)}="
      send(setter, value) if respond_to?(setter)
    end

    def set_attributes(hash)
      hash.each do |key, value|
        set_attribute(key, value)
      end
    end

    def parse_collection(ary, klass)
      ary.map { |obj|
        parse_object(obj, klass)
      }
    end

    def parse_object(obj, klass)
      if obj.is_a?(klass)
        obj
      else
        klass.new(obj)
      end
    end

    def parse_timestamp(value)
      return nil if value.nil?

      if value.is_a? Time
        value
      else
        if Time.respond_to? :zone
          Time.zone.parse(value)
        else
          Time.parse(value)
        end
      end
    end

    def inspect
      "#<#{self.class} #{[@id, @name || @display_name].compact.join(" ")}>"
    end
  end
end
